/**
 * @function setupMysql
 */

"use strict";

const execcli = require('execcli'),
    argx = require('argx'),
    async = require('async'),
    colorprint = require('colorprint'),
    assert = require('assert');

/** @lends setupMysql */
function setupMysql(env, options, callback) {
    let args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};

    const ROOT_USERNAME = env.ROOT_USERNAME || process.env.ROOT_USERNAME || 'root',
        ROOT_PASSWORD = env.ROOT_PASSWORD || process.env.ROOT_PASSWORD || process.env.MYSQL_ROOT_PASSWORD || '';


    const HOST = env.HOST || 'localhost',
        USERNAME = env.USER || env.USERNAME,
        DATABASE = env.DATABASE || env.SCHEMA,
        PASSWORD = env.PASSWORD;

    let sql = _sql(DATABASE, HOST, USERNAME, PASSWORD);

    let cmdBin = 'mysql',
        cmdArgs = _args(ROOT_USERNAME, HOST, ROOT_PASSWORD, sql);

    async.series([
        (callback) => {
            colorprint.debug(`  Executing SQL...`);
            colorprint.trace(sql.split(/;\s?/g).join(';\n'));
            callback(null);
        },
        (callback) => {
            execcli(cmdBin, cmdArgs, {}, callback);
        },
        (callback) => {
            colorprint.debug('  ...SQL done!');
            colorprint.debug('');
            callback(null);
        }
    ], callback);


}

function _sql(database, host, username, password) {
    return `CREATE DATABASE IF NOT EXISTS \`${database}\`; GRANT ALL ON \`${database}\`.* TO \`${username}\` IDENTIFIED BY "${password}"`;
}

function _args(username, host, password, sql) {
    let args = [
        '-u', username
    ];
    if (host) {
        args = args.concat('-h', host);
    }
    if (password) {
        process.env.MYSQL_PWD = password; //Use MYSQL_PWD to suppress insecure warning.
        //args = args.concat('-p' + password);
    }
    args = args.concat({'e': sql});
    return args;
}

module.exports = setupMysql;
